%% Aliasing
clear;
freq = 60;  
tmin = -0.05;
tmax = 0.05;
b = linspace(tmin, tmax, 1000);
signal = cos(2*pi*freq*b);
f = linspace(-5000,4990,1000);

%------------------------
% Change sampling period
%------------------------
T1 = 1/400;
%------------------------
nmin = ceil(tmin / T1);
nmax = floor(tmax / T1);
n1 = nmin:nmax;
x1 = cos(2*pi*freq*n1*T1);
f1 = linspace(-length(n1)/0.2+5,length(n1)/0.2-5,length(n1));

%------------------------
% Change sampling period
%------------------------
T2 = 1/70;
%------------------------
nmin = ceil(tmin / T2);
nmax = floor(tmax / T2);
n2 = nmin:nmax;
x2 = cos(2*pi*freq*n2*T2);
f2 = linspace(-length(n2)/0.2+5,length(n2)/0.2-5,length(n2));

figure(1)
subplot(2,3,1); plot(b,signal);
hold on; plot(n1*T1,x1,'.'); plot(n2*T2, x2, 'o'); hold off
title(sprintf('"Real World" Sampling shown\nwith dots and circles'));
xlabel('Time (ms)'); ylabel('Amplitude');
subplot(2,3,4); plot(f,abs(fftshift(fft(signal))))
xlabel('Frequency (Hz)'); ylabel('Amplitude');
subplot(2,3,2); plot(n1*T1,x1); 
title(['Sampled at ' num2str(1/T1) 'Hz (dots)']); xlabel('Time (ms)');
subplot(2,3,5); plot(f1,abs(fftshift(fft(x1))))
xlabel('Frequency (Hz)');
subplot(2,3,3); plot(n2*T2,x2); 
title(['Sampled at ' num2str(1/T2) 'Hz (circles)']); xlabel('Time (ms)');
subplot(2,3,6); plot(f2,abs(fftshift(fft(x2))))
xlabel('Frequency (Hz)');

%% Waveform
clear;
T = 0.001;
t = 0:T:.25;
ns = length(t);
load waveform;

%---------------------------------
% Your code here
%---------------------------------
Fs = 1/T;        % sampling frequency
Y = fftshift(fft(y))/sqrt(ns);         % Fourier transform of signal
hf = Fs/2;        % highest frequency measured
df = Fs/(ns-1);        % frequency step
f = -hf:df:hf;         % frequency vector
PSD = Y.*conj(Y);       % power spectral density
%---------------------------------

figure(2)
subplot(2,1,1); plot(1000*t,y); axis([0 250 -6 6])
title('Noisy Time Domain Signal'); xlabel('Time (ms)'); ylabel('Amplitude')
subplot(2,2,3); plot(f,abs(Y)); axis([-500 500 0 8])
title('Signal Spectrum'); xlabel('Frequency (Hz)'); ylabel('Amplitude')
subplot(2,2,4); plot(f,PSD); axis([0 500 0 60])
title('Signal Power Density Spectrum'); xlabel('Frequency (Hz)'); 
ylabel('Amplitude')

% figure(9)
% y = sin(2*pi*50*t) + sin(2*pi*120*t);
% Y = fftshift(fft(y,10000)/100);
% PSD = Y.*conj(Y);
% subplot(1,2,1); plot(0:250,y)
% subplot(1,2,2); plot(0:0.1:500,PSD(5000:end))

%% Two Sounds
clear;
%-----------------------------------
% Choose frequency window to cut out
%-----------------------------------
lowest = 200;
highest = 370;
%-----------------------------------

load train
n = 4*length(y);
b = interp1(1:4:n+1,[y',0],1:n)';
load handel
d  = y(1:n);
mix = d+b;
Fs = 10000; 
%-----------------------------------------
% Uncomment this to listen to the original
%-----------------------------------------
% sound(mix,Fs) 
%-----------------------------------------

Mix = fftshift(fft(mix));
f = linspace(-Fs/2,Fs/2-(Fs/n),n);
t = (0:n-1)/Fs;

notch = (or(abs(f)<lowest,abs(f)>highest))';
dprime = Mix.*notch;
dprime = ifft(ifftshift(dprime));
%-------------------------------------------------
% Uncomment this to listen to the processed signal
%-------------------------------------------------
% sound(dprime,Fs) 
%-------------------------------------------------

figure(3)
subplot(3,2,2); plot(t,d); axis([0, 5.5,-1,1])
title('Original Handel Music');
subplot(3,2,4); plot(t,mix); axis([0, 5.5,-1.5,1.5])
title('Mixed Music and Fog Horn');
subplot(1,2,1); plot(f,abs(Mix)); axis([0, 5000,0,1000])
title('Spectrum of Mixed Sound'); xlabel('Frequency (Hz)'); ylabel('Amplitude');
subplot(3,2,6);plot(t,dprime); axis([0, 5.5,-1,1])
title('Filtered Signal'); xlabel('Time (s)');

%% Hilo
clear;
%--------------------
% Choose filter width
%--------------------
sd = 8;
%--------------------
D = mat2gray(mean(imread('Hilo.jpg'),3));
Dhat = fftshift(fft2(D));
[d,y] = meshgrid(-(size(D,2)-1)/2:(size(D,2)-1)/2,-(size(D,1)-1)/2:(size(D,1)-1)/2);
[~,r] = cart2pol(d,y);
lpass = mat2gray(gaussmf(r,[sd,0]));
hpass = 1-lpass;
monroe = ifft2(ifftshift(lpass.*Dhat));
einstein = ifft2(ifftshift(hpass.*Dhat));

figure(4)
subplot(1,4,1)
imshow(D,[]); title('Original Illusion')
subplot(1,4,2)
imshow(log(abs(Dhat)),[]); title('FT of Original (log scale)')
subplot(1,4,3)
imshow(monroe,[]); title('Low Frequencies')
subplot(1,4,4)
imshow(einstein); title('High Frequencies')

%% Apodisation
clear;
%-----------------------------------------------------------------
% Change filter width (press any key after plotting to see result)
%-----------------------------------------------------------------
width = 50; %10;
%-----------------------------------------------------------------
d=1;
L=100;
lambda=1;
k=2*pi/lambda;

[x,y] = meshgrid(-500:500,-500:500);
[~,r] = cart2pol(x,y);
r(r==0) = lambda/1000;

I=(2*besselj(1,k*d*r/L)./(k*d*r./L));
Ihat = fftshift(fft2(I));

maskG = mat2gray(gaussmf(r,[width/2,0]));
maskTH = r<width;

Ihat1 = maskG.*Ihat;
Ihat2 = maskTH.*Ihat;
Iprime1 =(ifft2(Ihat1));
Iprime2 =(ifft2(Ihat2));

figure(5); 
subplot(2,2,1)
imshow(I,[]); title('Original Image')
subplot(2,2,2) 
imshow(log(abs(Ihat)),[]); title('FT of Original (log scale)')
subplot(2,2,3)
imshow(maskG,[]); title('Gaussian Filter')
subplot(2,2,4)
imshow(maskTH,[]); title('Top Hat Filter')
pause();
subplot(2,2,3)
imshow(abs(Iprime1),[]); title('Processed Gaussian Image')
subplot(2,2,4)
imshow(abs(Iprime2),[]); title('Processed Top Hat Image')

%% Deconvolution
clear;
npix = 16; sd = 3;
d = mat2gray(mean(imread('cameraman.tif'),3));
psf = fspecial('gaussian',npix,sd);
df1 = mat2gray(imfilter(d,psf));

n = 256;
psfmat = zeros(n);
psfmat((n-npix)/2+1:(n+npix)/2,(n-npix)/2+1:(n+npix)/2) = psf;
PSF = fft2(psfmat);
PSF(abs(PSF)==0) = 1e-9;

D = fft2(d);
df = mat2gray(ifftshift(ifft2(D.*PSF)));
df(9:248,9:248) = df1(9:248,9:248);

Df = fft2(fftshift(df));
rec = mat2gray(ifft2(Df./PSF));

figure(6)
subplot(2,2,1); imshow(d);
title('Original Image');
subplot(2,2,2); imshow(psfmat,[]);
title('Point Spread Function'); pause();
subplot(2,2,3); imshow(df1);
title('Convoluted Image'); pause();
subplot(2,2,4); imshow(rec);
title('Deconvoluted in the Fourier Domain');

%% SLM
clear;
load SLMmask
mask = mat2gray(mask);
%---------------------------------
% Your code here
%---------------------------------
in = exp(2i*pi*mask);          % Input side of the lens
out = fftshift(fft2(in));         % Output (Fourier) side of the lens
%---------------------------------
figure(7)
subplot(1,2,1); imshow(mask,[])
title('Phase Mask');
subplot(1,2,2); imshow(abs(out),[])
title('Ouput Amplitude')

%% Fourier Basis
figure(8)
subplot(1,3,1)
imagesc(eye(24));
axis square
subplot(1,3,2);
imagesc(real(sqrt(1/24)*dftmtx(24)));
axis square
subplot(1,3,3);
imagesc(imag(sqrt(1/24)*dftmtx(24)));
axis square