#-------------fft_slow.jl------------------------------------------------------#
#
# Purpose: Implement a slow version of an fft, to see how it's done.
#          For this, we will be using the Cooley-Tukey algorithm
#
#------------------------------------------------------------------------------#
using PyPlot

# Implementing the Cooley-Tukey Algorithm
function cooley_tukey(x)

    # First, we find the length of the input vector
    N = length(x)

    # Now we define the threshold at which we will start doing the slow DFT
    threshold = 4

    # If the input vector is not divisible by 2, end the program. 
    # We can only attack powers of 2 here.
    if(N%2 !=0)
        println("Number of elements must be divisible by 2!")
        exit(0)
    end

    # We choose this threshold arbitrarily
    if(N <= threshold)
        return DFT_slow(x)

    # if we have more elements than the threshold, we create two vectors and 
    # recursively split again and again
    else

        # x_even and x_odd will be divided up
        x_even = cooley_tukey(x[1:2:N])
        x_odd = cooley_tukey(x[2:2:N])

        # generating the factor for splitting
        k = 0:N-1
        half = N/2
        factor = exp(-2im*pi*k/N)

        # Concatenating
        return vcat(x_even + factor[1:half] .* x_odd,
                     x_even + factor[half+1:N] .* x_odd) 
    end
    
end

# A super slow DFT to start
function DFT_slow(x)
    N = length(x)

    # range is automatically a column vector, and we want a row vector
    n = 0:N-1
    k = n'
    M = exp(-2im * pi *n *k / N)
    return M * x
end

x = rand(4)
println(cooley_tukey(x) - fft(x))
println(isapprox(cooley_tukey(x), fft(x)))

#=
x = sin(1:4)
plot(x)
plot(abs(DFT_slow(x)))
=#
