%{-------------split_op.m------------------------------------------------------%
%
% Purpose: Simple matlab code for the split_operator method
%
%   Notes: The method here involves splitting the Schroedinger Eq into two
%          terms and multiplying by one term in real space and another term in
%          fourier space. This allows us to perform a derivative in time and
%          space, necessary to solve the equation.
%
%------------------------------------------------------------------------------%

% Defining all the necessary variables and subfunctions

xmax = 10;
res = 512;
dt = 0.05;
timesteps = 1000;
dx = 2*xmax/res;
x = linspace(-xmax+dx,xmax,res);
dk = pi / xmax;
k = [0:res/2-1,-res/2:-1]*dk;

% Function to compute the potential for the split-operator method
% In this case, we are assuming a simple harmonic potential for V
voffset = 0;
V = 0.5*(x-voffset).^2 ;
%V = 0.5*(x-voffset).^2 +10*exp(-x.^2/4);
%V = 5*cos(pi*x/5);


% Function to initialize the wfc as a Gaussian
x0= 0; % horizontal offset
sigma = sqrt(2); % width of the initial wfc
wfc = exp(-(x-x0).^2/sigma^2);
wfc = wfc/sqrt(sum(abs(wfc).^2*dx));
density = zeros(res,1);

% Function to find the potential and kinetic energy terms for split-op
% First, we need to initialize k, note that this split in the middle

KE = exp(-0.5*1i*k.^2*dt);
PE = exp(-0.5*1i*V*dt);

% Now for the simulation loop. In each step, we will multiply by one
% parameter in real space, move to Fourier Space, multiply it by
% another variable, and then transform back

for ii = 1:timesteps
    % In the case, we are solving a non-linear Schroedinger equation,
    % our KE and PE terms would depend on the wfc which is changing
    % every timestep, so we would update those terms here like so
    %    PE,KE = energy();
    
    % We multiply in real space
    wfc = wfc.*PE;
    
    % now we fft into phase space
    wfc = fft(wfc);
    
    % and multiply by KE
    wfc = wfc.*KE;
    
    % then ifft back
    wfc = ifft(wfc);
    
    % One last multiplication in real space
    wfc = wfc.*PE;
    
    % Now let's grab the wfc density and plot it
    density = abs(wfc).^2;
    clf();
    plot(x,density,x,V/max(V)*1.5)
    axis([-xmax,xmax,0,1.5])
    pause(0.05)
    drawnow
    
end

